@@ -63,28 +63,32 @@ class AttributeAddQuery(Query):
         MATCH p = (n:%(node_kind)s)
         CALL {
             WITH n
-            MATCH (root:Root)<-[r1:IS_PART_OF]-(n)
-            OPTIONAL MATCH (n)-[r2:HAS_ATTRIBUTE]-(:Attribute { name: $attr_name })
-            WHERE all(r in [r1, r2] WHERE (%(branch_filter)s))
-            RETURN n as n1, r1 as r11, r2 as r12
-            ORDER BY r2.branch_level DESC, r2.from ASC, r1.branch_level DESC, r1.from ASC
+            MATCH (:Root)<-[r:IS_PART_OF]-(n)
+            WHERE %(branch_filter)s
+            WITH n, r AS is_part_of_e
+            OPTIONAL MATCH (n)-[r:HAS_ATTRIBUTE]-(:Attribute { name: $attr_name })
+            WHERE %(branch_filter)s
+            WITH is_part_of_e, r AS has_attr_e
+            RETURN is_part_of_e, has_attr_e
+            ORDER BY has_attr_e.branch_level DESC, has_attr_e.from ASC, is_part_of_e.branch_level DESC, is_part_of_e.from ASC
             LIMIT 1
         }
-        WITH n1 as n, r11 as r1, r12 as r2, av, is_protected_value, is_visible_value
-        WHERE r1.status = "active" AND (r2 IS NULL OR r2.status = "deleted")
+        WITH n, is_part_of_e, has_attr_e, av, is_protected_value, is_visible_value
+        WHERE is_part_of_e.status = "active" AND (has_attr_e IS NULL OR has_attr_e.status = "deleted")
         CREATE (a:Attribute { name: $attr_name, branch_support: $branch_support })
         CREATE (n)-[:HAS_ATTRIBUTE $rel_props ]->(a)
         CREATE (a)-[:HAS_VALUE $rel_props ]->(av)
         CREATE (a)-[:IS_PROTECTED $rel_props]->(is_protected_value)
         CREATE (a)-[:IS_VISIBLE $rel_props]->(is_visible_value)
         %(uuid_generation)s
-        FOREACH (i in CASE WHEN r2.status = "deleted" THEN [1] ELSE [] END |
-            SET r2.to = $current_time
+        FOREACH (i in CASE WHEN has_attr_e.status = "deleted" THEN [1] ELSE [] END |
+            SET has_attr_e.to = $current_time
         )
         """ % {
             "branch_filter": branch_filter,
             "node_kind": self.node_kind,
             "uuid_generation": db.render_uuid_generation(node_label="a", node_attr="uuid"),
         }
+
         self.add_to_query(query)
         self.return_labels = ["n.uuid", "a.uuid"]
