@@ -155,7 +155,6 @@ class SchemaManager(NodeManager):
 
         updated_schema = None
         if update_db:
-            schema_diff = None
             if diff:
                 schema_diff = await self.update_schema_to_db(schema=schema, db=db, branch=branch, diff=diff)
             else:
@@ -295,15 +294,21 @@ class SchemaManager(NodeManager):
             new_node.attributes = []
 
             for item in node.attributes:
-                new_attr = await self.create_attribute_in_db(
-                    schema=attribute_schema, item=item, parent=obj, branch=branch, db=db
-                )
+                if item.inherited is False:
+                    new_attr = await self.create_attribute_in_db(
+                        schema=attribute_schema, item=item, parent=obj, branch=branch, db=db
+                    )
+                else:
+                    new_attr = item.duplicate()
                 new_node.attributes.append(new_attr)
 
             for item in node.relationships:
-                new_rel = await self.create_relationship_in_db(
-                    schema=relationship_schema, item=item, parent=obj, branch=branch, db=db
-                )
+                if item.inherited is False:
+                    new_rel = await self.create_relationship_in_db(
+                        schema=relationship_schema, item=item, parent=obj, branch=branch, db=db
+                    )
+                else:
+                    new_rel = item.duplicate()
                 new_node.relationships.append(new_rel)
 
         # Save back the node with the newly created IDs in the SchemaManager
@@ -465,7 +470,8 @@ class SchemaManager(NodeManager):
 
         if diff_attributes:
             for item in node.local_attributes:
-                if item.name in diff_attributes.added:
+                # if item is in changed and has no ID, then it is being overridden from a generic and must be added
+                if item.name in diff_attributes.added or item.name in diff_attributes.changed and item.id is None:
                     created_item = await self.create_attribute_in_db(
                         schema=attribute_schema, item=item, branch=branch, db=db, parent=obj
                     )
@@ -484,7 +490,8 @@ class SchemaManager(NodeManager):
 
         if diff_relationships:
             for item in node.local_relationships:
-                if item.name in diff_relationships.added:
+                # if item is in changed and has no ID, then it is being overridden from a generic and must be added
+                if item.name in diff_relationships.added or item.name in diff_relationships.changed and item.id is None:
                     created_rel = await self.create_relationship_in_db(
                         schema=relationship_schema, item=item, branch=branch, db=db, parent=obj
                     )
@@ -715,6 +722,9 @@ class SchemaManager(NodeManager):
             for rel in await rm.get(db=db):
                 item = await rel.get_peer(db=db)
                 item_data = {"id": item.id}
+                inherited_attr = getattr(item, "inherited", None)
+                if inherited_attr and getattr(inherited_attr, "value", False) is True:
+                    continue
                 for item_name in item._attributes:
                     item_attr = getattr(item, item_name)
                     item_data[item_name] = item_attr.get_value()
@@ -733,3 +743,24 @@ class SchemaManager(NodeManager):
         """Convert a schema_node object loaded from the database into GenericSchema object."""
         node_data = await cls._prepare_node_data(schema_node=schema_node, db=db)
         return GenericSchema(**node_data)
+
+    def purge_inactive_branches(self, active_branches: list[str]) -> list[str]:
+        """Return non active branches that were purged."""
+
+        hashes_to_keep: set[str] = set()
+        for active_branch in active_branches:
+            if branch := self._branches.get(active_branch):
+                nodes = branch.get_all(include_internal=True, duplicate=False)
+                hashes_to_keep.update([node.get_hash() for node in nodes.values()])
+
+        removed_branches: list[str] = []
+        for branch_name in list(self._branches.keys()):
+            if branch_name not in active_branches:
+                del self._branches[branch_name]
+                removed_branches.append(branch_name)
+
+        for hash_key in list(self._cache.keys()):
+            if hash_key not in hashes_to_keep:
+                del self._cache[hash_key]
+
+        return removed_branches
