@@ -1,5 +1,6 @@
 from pydantic import Field
 
+from infrahub.context import InfrahubContext
 from infrahub.message_bus import InfrahubMessage
 
 
@@ -10,3 +11,5 @@ class FinalizeValidatorExecution(InfrahubMessage):
     validator_execution_id: str = Field(..., description="The id of current execution of the associated validator")
     start_time: str = Field(..., description="Start time when the message was first created")
     validator_type: str = Field(..., description="The type of validator to complete")
+    context: InfrahubContext = Field(..., description="The Infrahub context")
+    proposed_change: str = Field(..., description="The ID of the proposed change")
