@@ -4,7 +4,6 @@ from prefect import Flow
 from infrahub.message_bus import RPCErrorResponse, messages
 from infrahub.message_bus.operations import (
     check,
-    event,
     finalize,
     git,
     refresh,
@@ -17,8 +16,6 @@ from infrahub.tasks.check import set_check_status
 
 COMMAND_MAP = {
     "check.generator.run": check.generator.run,
-    "event.branch.merge": event.branch.merge,
-    "event.worker.new_primary_api": event.worker.new_primary_api,
     "finalize.validator.execution": finalize.validator.execution,
     "git.file.get": git.file.get,
     "git.repository.connectivity": git.repository.connectivity,
